<%-- $Id: list_quota.jsp 7939 2021-05-04 07:30:31Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Quota"
	import="net.sf.basedb.core.QuotaType"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Location"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.util.Enumeration"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ModeInfo"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnAction"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="java.util.List"
	import="java.util.Map"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.QUOTA;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.LIST);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.DENIED, itemType);
final String ID = sc.getId();
final boolean createPermission = sc.hasPermission(Permission.CREATE, itemType);
final boolean writePermission = sc.hasPermission(Permission.WRITE, itemType);
final boolean deletePermission = sc.hasPermission(Permission.DELETE, itemType);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);

final ModeInfo mode = ModeInfo.get(request.getParameter("mode"));
final String callback = request.getParameter("callback");
final String title = mode.generateTitle("quota", "quota");
final DbControl dc = sc.newDbControl();
ItemResultIterator<Quota> quota = null;
try
{
	final QuotaType total = QuotaType.getById(dc, SystemItems.getId(QuotaType.TOTAL));

	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, null);
	try
	{
		final ItemQuery<Quota> query = Base.getConfiguredQuery(dc, cc, jspContext, true, Quota.getQuery(), mode);
		quota = query.iterate(dc);
	}
	catch (Throwable t)
	{
		cc.setMessage(t.getMessage());
	}
	int numListed = 0;
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	ExtensionsInvoker<ListColumnAction<Quota,?>> columnsInvoker = ListColumnUtil.useExtensions(jspContext);
	%>
	<base:page title="<%=title==null ? "Quota" : title%>" type="<%=mode.getPageType()%>" id="list-page">
	<base:head scripts="table.js,~quota.js" styles="table.css,toolbar.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	
	<base:body>
		<h1><%=title==null ? "Quota" : title%></h1>
		<div class="content">
		<tbl:table 
			id="quota" 
			columns="<%=cc.getSetting("columns")%>"
			sortby="<%=cc.getSortProperty()%>" 
			direction="<%=cc.getSortDirection()%>"
			action="index.jsp"
			sc="<%=sc%>"
			item="<%=itemType%>"
			filterrows="<%=cc.getFilterRows()%>"
			subclass="fulltable"
			stickyheaders="name"
			>
			<tbl:hidden 
				name="mode" 
				value="<%=mode.getName()%>" 
			/>
			<tbl:hidden 
				name="callback" 
				value="<%=callback%>" 
				skip="<%=callback == null%>" 
			/>
			<tbl:columndef 
				id="name"
				property="name"
				datatype="string"
				title="Name"
				sortable="true" 
				filterable="true"
				exportable="true"
				show="always" 
			/>
			<tbl:columndef 
				id="id"
				clazz="uniquecol"
				property="id"
				datatype="int"
				title="ID"
				sortable="true"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="systemId"
				clazz="uniquecol"
				property="systemId"
				datatype="string"
				title="System ID"
				sortable="true"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="total"
				title="Total quota"
			/>
			<tbl:columndef 
				id="description"
				property="description"
				datatype="string"
				title="Description" 
				sortable="true" 
				filterable="true" 
				exportable="true"
			/>
			<tbl:columndef 
				id="xt-columns" 
				extensions="<%=columnsInvoker%>" 
				jspcontext="<%=jspContext%>" 
			/>
			<div class="panelgroup bg-filled-50 bottomborder">
				<tbl:toolbar
					subclass="bottomborder"
					visible="<%=mode.hasToolbar()%>"
					>
					<tbl:button 
						id="btnNewItem"
						disabled="<%=!createPermission%>" 
						image="new.png" 
						title="New&hellip;" 
						tooltip="<%=createPermission ? "Create new quota" : "You do not have permission to create quota"%>" 
					/>
					<tbl:button 
						id="btnDeleteItems"
						disabled="<%=!deletePermission%>" 
						image="delete.png" 
						title="Delete" 
						tooltip="<%=deletePermission ? "Delete the selected items" : "You do not have permission to delete quota" %>" 
					/>
					<tbl:button 
						id="btnRestoreItems"
						disabled="<%=!writePermission%>" 
						image="restore.png" 
						title="Restore" 
						tooltip="<%=writePermission ? "Restore the selected (deleted) items" : "You do not have permission to edit quota" %>" 
					/>
					<tbl:button 
						id="btnColumns"
						image="columns.png" 
						title="Columns&hellip;" 
						tooltip="Show, hide and re-order columns" 
					/>
					<tbl:button 
						id="btnImport"
						data-plugin-type="IMPORT"
						image="import.png" 
						title="Import&hellip;" 
						tooltip="Import data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
					/>
					<tbl:button 
						id="btnExport"
						data-plugin-type="EXPORT"
						image="export.png" 
						title="Export&hellip;" 
						tooltip="Export data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
					/>
					<tbl:button 
						id="btnRunPlugin"
						data-plugin-type="OTHER"
						image="runplugin.png" 
						title="Run plugin&hellip;" 
						tooltip="Run a plugin" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
					/>
					<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
						wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
				</tbl:toolbar>
				<tbl:panel>
					<tbl:presetselector />
					<tbl:navigator
						page="<%=cc.getPage()%>" 
						rowsperpage="<%=cc.getRowsPerPage()%>" 
						totalrows="<%=quota == null ? 0 : quota.getTotalCount()%>" 
						visible="<%=mode.hasNavigator()%>"
					/>
				</tbl:panel>
			</div>
			<tbl:data>
				<tbl:headers>
					<tbl:headerrow>
						<tbl:header clazz="row-index bg-filled-100" />
						<tbl:columnheaders />
					</tbl:headerrow>
					<%
					int numFilters = cc.getNumPropertyFilters();
					int numRows = cc.getFilterRows();
					for (int filterNo = 0; filterNo < numRows; filterNo++)
					{
						boolean lastRow = filterNo == numRows-1;
						%>
						<tbl:headerrow>
							<tbl:header subclass="row-index bg-filled-100">
								<div class="index-<%=mode.getName()%>">
									<div class="index"></div>
									<div class="check">
										<base:icon 
											subclass="link table-check"
											image="check_uncheck.png" 
											tooltip="Toggle all (use CTRL, ALT or SHIFT to check/uncheck)" 
											visible="<%=lastRow && mode.hasCheck()%>"
										/>
									</div>
									<div class="icons">
										<base:icon
											subclass="link table-filter-row-action"
											image="add.png"
											tooltip="Add extra filter row"
											visible="<%=lastRow%>"
										/><base:icon
											subclass="link table-filter-row-action"
											image="remove.png"
											tooltip="Remove this filter row"
											visible="<%=numRows > 1 || numFilters > 0 %>"
											data-remove-row="<%=filterNo%>"
										/>
									</div>
								</div>
							</tbl:header>
							<tbl:propertyfilter row="<%=filterNo%>" />
						</tbl:headerrow>
						<%
					}
					%>
					<tbl:columnsubtitles />
				</tbl:headers>
				<tbl:rows>
					<%
					if (cc.getMessage() != null)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer error"><%=cc.getMessage()%></div>
						</tbl:panel>
						<%
						cc.setMessage(null);
					}
					int index = cc.getPage()*cc.getRowsPerPage();
					int selectedItemId = cc.getId();
					if (quota != null)
					{						
						while (quota.hasNext())
						{
							Quota item = quota.next();
							int itemId = item.getId();
							String name = HTML.encodeTags(item.getName());
							String tooltip = mode.isSelectionMode() ? 
									"Select this item" : "View this item" + (writePermission ? " (use CTRL, ALT or SHIFT to edit)" : "");
							
							index++;
							numListed++;
							long totalBytes = item.getQuotaValue(total, Location.PRIMARY);
							%>
							<tbl:row>
								<tbl:header clazz="row-index bg-filled-100">
									<div class="index-<%=mode.getName()%>">
										<div class="index <%=index>999?"index-smaller":""%>"><%=index%></div>
										<div class="check">
											<base:input
												type="checkbox" 
												name="<%=itemId%>" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												checked="<%=cc.getSelected().contains(itemId)%>"
												visible="<%=mode.hasCheck()%>"
											/>
											<base:input 
												type="radio" 
												name="item_id" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												checked="<%=selectedItemId == itemId%>"
												visible="<%=mode.hasRadio()%>"
											/>
										</div>
										<div class="icons">
											<base:icon 
												image="deleted.png"
												id="<%="delete."+itemId %>"
												subclass="<%=deletePermission ? "table-delete-item" : "disabled" %>"
												data-item-id="<%=itemId%>"
												tooltip="This item has been scheduled for deletion" 
												visible="<%=item.isRemoved()%>"
											/><base:icon
												image="systemitem.png"
												tooltip="This item is a system item"
												visible="<%=item.isSystemItem()%>" 
											/>
										</div>
									</div>
								</tbl:header>
								<tbl:cell column="name"><div 
									class="link table-item"
									data-item-id="<%=itemId%>"
									data-no-edit="<%=writePermission ? 0 : 1 %>" 
									tabindex="0"
									title="<%=tooltip%>"><%=name%></div></tbl:cell>
								<tbl:cell column="id"><%=item.getId()%></tbl:cell>
								<tbl:cell column="systemId"><%=Values.getString(item.getSystemId())%></tbl:cell>
								<tbl:cell column="total"><%=totalBytes == Quota.UNLIMITED ? "<i>- unlimited -</i>" : Values.formatBytes(totalBytes)%></tbl:cell>
								<tbl:cell column="description"><%=HTML.encodeTags(item.getDescription())%></tbl:cell>
								<tbl:xt-cells dc="<%=dc%>" item="<%=item%>">
									<tbl:cell column="xt-columns" />
								</tbl:xt-cells>
							</tbl:row>
							<%
							}
						}
					if (numListed == 0)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer note">
							<%=quota == null || quota.getTotalCount() == 0 ? "No quota were found" : "No quota on this page. Please select another page!" %>
							</div>
						</tbl:panel>
						<%
					}
					%>
				</tbl:rows>
			</tbl:data>
		</tbl:table>
		</div>
		
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnOk" title="Ok" visible="<%=mode.hasOkButton()%>" />
			<base:button id="close" title="Cancel" visible="<%=mode.hasCancelButton()%>" />
			<base:button id="close" title="Close" visible="<%=mode.hasCloseButton()%>" />
		</base:buttongroup>
		
	</base:body>
	</base:page>
	<%
}
finally
{
	if (quota != null) quota.close();
	if (dc != null) dc.close();
}
%>
