<%-- $Id: jep_extra_value_calculator.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.BioAssay"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.RawDataType"
	import="net.sf.basedb.core.BioAssaySet"
	import="net.sf.basedb.core.Formula"
	import="net.sf.basedb.core.Job"
	import="net.sf.basedb.core.ExtraValueType"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"	
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.util.json.NameableConverter"
	import="org.json.simple.JSONObject"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%!
private static void addFormulaOption(StringBuilder options, String formula, String title, String description)
{
	options.append("<option value=\"").append(HTML.encodeTags(formula)).append("\"");
	options.append(" title=\"").append(HTML.encodeTags(description)).append("\"");
	options.append(">").append(HTML.encodeTags(title)).append("\n");
}
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
ItemContext cc = sc.getCurrentContext(Item.BIOASSAYSET);
ItemContext realContext = sc.getSessionSetting("plugin.configure.currentContext");
if (realContext == null) realContext = cc;
DbControl dc = null;
try
{
	String errorMessage = sc.getSessionSetting("plugin.configure.errors.message");
	List<Throwable> errors = sc.getSessionSetting("plugin.configure.errors.list");

	dc = sc.newDbControl();
	Job job = sc.getSessionSetting("plugin.configure.job");
	dc.reattachItem(job, false);
	BioAssaySet source = null;
	if (job != null) source = job.getParameterValue("source");
	if (source == null && cc.getId() != 0) source = BioAssaySet.getById(dc, cc.getId());
	if (source == null) throw new WebException("popup", "No current bioassay set", 
		"Could not find any current bioassay set. Please select a bioassay set before trying again");
	source = BioAssaySet.getById(dc, source.getId());
	int maxRawMappings = source.getMaxRawMappingsForSpot();

	RawDataType rdt = source.getRawDataType();
	
	//Find selected bioassays
	List<BioAssay> bioAssays = null;
	if (job != null) bioAssays = job.getParameterValues("bioAssays");
	if (bioAssays == null)
	{
		ItemContext bioAssayContext = sc.getCurrentContext(Item.BIOASSAY);
		ItemQuery<BioAssay> bioAssayQuery = null;
		if (bioAssayContext.getSelected().size() > 0)
		{
			bioAssayQuery = source.getBioAssays();
			bioAssayQuery.restrict(
				Restrictions.in(Hql.property("id"), Expressions.parameter("selected"))
			);
			bioAssayQuery.setParameter("selected", bioAssayContext.getSelected(), Type.INT);
			bioAssays = bioAssayQuery.list(dc);
		}
	}
	
	JSONObject jsonBioAssays = new JSONObject();
	jsonBioAssays.put("itemType", "BIOASSAY");
	if (bioAssays != null)
	{
		jsonBioAssays.put("items", JsonUtil.toArray(bioAssays, new NameableConverter<BioAssay>()));
	}

	// Current parameter values
	ExtraValueType currentExtraValueType = job.getParameterValue("extraValueType");
	String expression = job.getParameterValue("expression");
	
	
	// Predefined formulas
	ItemQuery<Formula> formulaQuery = Formula.getQuery(Formula.Type.COLUMN_EXPRESSION, rdt, 
			source.getIntensityTransform());
	formulaQuery.order(Orders.asc(Hql.property("name")));
	formulaQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
	ItemResultList<Formula> formulas = formulaQuery.list(dc);
	StringBuilder formulaOptions = new StringBuilder();
	for (Formula f : formulas)
	{
		String formula = f.getFormula(0);
		if (maxRawMappings == 1 || !formula.contains("raw("))
		{
			addFormulaOption(formulaOptions, formula, f.getName(), f.getDescription());
		}
	}
	
	// Load recently used items
	List<ExtraValueType> recentExtraValueTypes = realContext.getRecent(dc, Item.EXTRAVALUETYPE);
	%>
	<base:page type="popup" title="Calculate extra value">
	<base:head scripts="linkitems-2.js,/plugins/net/sf/basedb/plugins/jep_plugins.js,/plugins/net/sf/basedb/plugins/jep_extra_value_calculator.js" />
	<base:body>
		<h1>Calculate extra value <base:help helpid="jepextravaluecalculator.options"/></h1>
		<div id="page-data" class="datacontainer"
			data-max-raw-mapping="<%=maxRawMappings %>"
		></div>
		<form name="extraValue" action="index.jsp" method="post">
		<input type="hidden" name="ID" value="<%=ID%>">
		<input type="hidden" name="requestId" value="<%=request.getParameter("requestId")%>">
		<input type="hidden" name="cmd" value="SetParameters">
		<input type="hidden" name="parameter:source" value="<%=source.getId()%>">
		
		<div class="content bottomborder">
			<table class="fullform input100">
			<%
			if (errorMessage != null || (errors != null && errors.size() > 0))
			{
				%>
				<tr>
					<th></th>
					<td colspan="2">
					<div id="errors" style="margin-bottom: 12px;">
						<div class="messagecontainer error" style="margin: 0px;">
						<%=errorMessage %>
						<%
						if (errors != null && errors.size() > 0)
						{
							%>
							<div id="showerrorlist">
								<base:icon 
									id="btnShowError"
									image="gonext.png" 
									style="color: #FFFFFF;"
									tooltip="Show more information about the error"
								/>
							</div>
							<div id="errorlist" style="display: none; margin: 0px;">
								<base:icon 
									id="btnHideError"
									image="move_down.png" 
									style="color: #FFFFFF;" 
									tooltip="Show less information"
								/>
								<ol>
								<%
								for (Throwable t : errors)
								{
									%>
									<li><%=t.getMessage()%>
									<%
								}
								%>
								</ol>
							</div>
							<%
						}
						%>
					</div>
					</td>
				</tr>
				<%
			}
			%>
			<tr>
				<th>Source</th>
				<td><%=HTML.encodeTags(source.getName())%></td>
				<td></td>
			</tr>
			<tr class="big">
				<th class="subprompt">Bioassays</th>
				<td>
					<input type="checkbox" name="allBioAssays" id="allBioAssays"
						><label for="allBioAssays">All bioassays</label><br>
					<div class="selectionlist">
						<table>
						<tr>
						<td>
							<select name="bioAssays" id="bioAssays" 
								class="auto-init"
								data-auto-init="link-container"
								data-initial-items="[<%=HTML.encodeTags(jsonBioAssays.toJSONString()) %>]"
								size="5" multiple>
							</select>
						</td>
						<td style="vertical-align: top;">
							<base:buttongroup vertical="true">
								<base:button 
									id="btnAddBioAssays"
									subclass="leftaligned" 
									title="Add bioassays&hellip;" 
									tooltip="Add bioassays"
								/>
								<base:button 
									id="btnRemoveBioAssays"
									subclass="leftaligned auto-init"
									data-auto-init="remove-link"
									data-list-id="bioAssays"
									title="Remove" 
									tooltip="Remove the selected bioassays"
								/>
							</base:buttongroup>
						</td>
						</tr>
						</table>
					</div>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>Extra value type</th>
				<td>
					<base:select 
						id="parameter:extraValueType"
						clazz="selectionlist"
						required="true"
						current="<%=currentExtraValueType%>"
						recent="<%=recentExtraValueTypes%>"
						newitem="true"
					/>
				</td>
				<td></td>
			</tr>
			
			<tr>
				<th>Expression</th>
				<td>
					<select name="presets" id="presets" style="width: 30em;"
						data-formula-id="parameterExpression">
						<option value="">- select from list or enter formula below -
						<%=formulaOptions.toString()%>
					</select>
				</td>
				<td></td>
			</tr>
			
			<tr class="big">
				<th class="subprompt"></th>
				<td>
					<textarea class="text required" name="parameter:expression" 
						id="parameterExpression" rows="3" ><%=HTML.encodeTags(expression)%></textarea>
				</td>
				<td style="width: 20px;">
					<base:icon 
						id="btnExpressionBuilder" 
						data-formula-id="parameterExpression"
						data-title="Expression"
						data-formula-type="COLUMN_EXPRESSION"
						data-raw-data-type="<%=rdt.getId() %>"
						data-channels="<%=rdt.getChannels() %>"
						data-bioassayset="<%=source.getId() %>"
						image="expression_builder.png"
						tooltip="A simple utility for building expressions"
					/>
				</td>
			</tr>
			<tr class="dynamic">
				<th></th>
				<td colspan="2"></td>
			</tr>
			</table>
		</div>
		</form>
	
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnNext" title="Next" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}%>
