<%-- $Id: view_job.jsp 7938 2021-05-03 11:20:19Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Job"
	import="net.sf.basedb.core.JobAgent"
	import="net.sf.basedb.core.Project"
 	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.Path"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.ChangeHistory"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"	
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.PluginConfiguration"
	import="net.sf.basedb.core.PluginType"
	import="net.sf.basedb.core.ParameterInfo"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.NonRestartable"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.signal.SignalTransporter"
	import="net.sf.basedb.core.signal.Signal"
	import="net.sf.basedb.plugins.util.Parameters"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ChangeHistoryUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="java.util.Date"
	import="java.util.Map"
	import="java.util.Set"
	import="java.util.List"
	import="java.util.Collections"
	import="java.util.Collection"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%!
	private static final Item itemType = Item.JOB;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.ITEM);

	private String getItemName(BasicItem item)
	{
		String itemName = "";
		if (item instanceof File)
		{
			itemName = ((File)item).getPath().toString();
		}
		else if (item instanceof Nameable)
		{
			itemName = ((Nameable)item).getName();
		}
		else
		{
			itemName = item.toString();
		}
		return itemName;
	}
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String tab = Values.getString(request.getParameter("tab"), "properties");
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	Job job = Job.getById(dc, itemId);
	String title = "View job -- " + HTML.encodeTags(job.getName());
	
	final boolean writePermission = job.hasPermission(Permission.WRITE);
	final boolean deletePermission = job.hasPermission(Permission.DELETE);
	final boolean isRemoved = job.isRemoved();
	final boolean isUsed = isRemoved && job.isUsed();
	final boolean deletePermanentlyPermission = deletePermission && !isUsed;
	
	boolean readPlugin = true;
	PluginDefinition plugin = null;
	try
	{
		plugin = job.getPluginDefinition();
	}
	catch (PermissionDeniedException ex)
	{
		readPlugin = false;
	}
	boolean readCurrentConfig = true;
	PluginConfiguration currentConfig = null;
	try
	{
		currentConfig = job.getPluginConfiguration();
	}
	catch (PermissionDeniedException ex)
	{
		readCurrentConfig = false;
	}
	boolean readAgent = true;
	JobAgent agent = null;
	try
	{
		agent = job.getJobAgent();
	}
	catch (PermissionDeniedException ex)
	{
		readAgent = false;
	}
	boolean readActiveProject = true;
	Project activeProject = null;
	try
	{
		if (job.getActiveProjectId() != 0) 
		{
			activeProject = Project.getById(dc, job.getActiveProjectId());
		}
	}
	catch (PermissionDeniedException ex)
	{
		readActiveProject = false;
	}
	catch (Exception ex)
	{
		// Some other problem (for example, no longer exists)
	}

	int parameterVersion = job.getParameterVersion();
	int latestParameterVersion = currentConfig == null ? parameterVersion : currentConfig.getParameterVersion();

	final Job.Status status = job.getStatus();
	final boolean autoUpdate = status == Job.Status.WAITING || 
		status == Job.Status.PREPARED || status == Job.Status.EXECUTING || status == Job.Status.ABORTING;
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	Set<String> jobParameters = job.getParameterNames();
	File logFile = null;
	if (!autoUpdate && jobParameters != null && jobParameters.contains(Parameters.LOGFILE_PARAMETER))
	{
		try
		{
			Object temp = job.getParameterValue(Parameters.LOGFILE_PARAMETER);
			if (temp instanceof File)
			{
				logFile = (File)temp;
			}
			else if (temp instanceof String)
			{
				logFile = File.getByPath(dc, new Path((String)temp, Path.Type.FILE), false);
			}
		}
		catch (Throwable t)
		{}
	}
	
	// Check if the job can be aborted or paused
	boolean supportsAbort = false; 
	boolean supportsPause = false;
	boolean supportsForceError = writePermission && status != Job.Status.DONE && status != Job.Status.ERROR;
	
	// The status must be executable or EXECUTING
	if ((status == Job.Status.EXECUTING || status.isExecutable()) && writePermission)
	{
		// Check if a signal-transporter exists
		try
		{
			SignalTransporter signalTransporter = job.getSignalTransporter();
			if (signalTransporter == null)
			{
				// The job is only abortable if it is waiting
				supportsAbort = status == Job.Status.WAITING;
			}
			else
			{
				// Otherwise we must check with the signal transporter
				Collection<Signal> supportedSignals = signalTransporter.getSupportedSignals();
				supportsAbort = supportedSignals == null || supportedSignals.contains(Signal.ABORT);
				supportsPause = supportedSignals != null && supportedSignals.contains(Signal.PAUSE);
			}
		}
		catch (Exception ex)
		{}
	}
	if (autoUpdate) job.requestStatusUpdate();
	%>
	<base:page type="popup" title="<%=title%>" id="view-page">	
	<base:head scripts="tabcontrol-2.js,table.js,~jobs.js" 
		styles="tabcontrol.css,progressbar.css,table.css">
	<style>
	.executecommand
	{
		font-size: 0.85em;
		color: #777777;
	}
	</style>
	</base:head>
	<base:body data-read-only="1">
		<h1><%=title%> <base:help tabcontrol="main" /></h1>
		<div id="page-data" 
			data-item-id="<%=itemId%>"
			data-job-parameter-version="<%=parameterVersion %>"
			data-latest-parameter-version="<%=latestParameterVersion %>"
			data-breakpoint="<%=HTML.encodeTags(job.getExecuteCommand()) %>"
			data-job-status="<%=status.name()%>"
			data-auto-update="<%=autoUpdate ? 1 : 0 %>"
		></div>

		<t:tabcontrol id="main" 
			subclass="content dialogtabcontrol"
			active="<%=tab%>" position="bottom">
		<t:tab id="properties" title="Properties" helpid="job.view.properties">
			<table class="fullform outlined">
			<%
			if (isRemoved)
			{
				%>
				<tr>
					<th></th>
					<td colspan="2">
					<div class="itemstatus">
						<base:icon 
							id="btnDeletePermanently"
							image="deleted.png"
							data-notify="btnDeletePermanently"
							enabled="<%=deletePermanentlyPermission %>"
							tooltip="<%=deletePermanentlyPermission ? "Permanently delete this item" : null%>"
							visible="<%=isRemoved%>"> This item has been flagged for deletion<br></base:icon>
						<base:icon
							id="btnUsingItems"
							image="used.png" 
							tooltip="Show the items that are using this one"
							visible="<%=isUsed%>"> This item is used by other items and can't be permanently deleted<br></base:icon>
					</div>
					</td>
				</tr>
				<%
			}
			%>
			<tr>
				<th>Name</th>
				<td><%=HTML.encodeTags(job.getName())%></td>
			</tr>
			<tr>
				<th>Plugin</th>
				<td>
					<base:propertyvalue item="<%=job%>" property="pluginDefinition.name" nulltext="<i>External</i>"/> 
					<%=job.getPluginVersion() == null ?
						"" : "(" + HTML.encodeTags(job.getPluginVersion()) + ")"%>
				</td>
			</tr>
			<%
			if (currentConfig != null)
			{
				%>
				<tr>
					<th class="subprompt">- configuration</th>
					<td>
						<base:propertyvalue item="<%=job%>" property="pluginConfiguration.name" />
					</td>
				</tr>
				<%
			}
			%>
			<tr>
				<th>Type</th>
				<td>
					<%=plugin == null ? "" : plugin.getMainType() %>
					<base:propertyvalue item="<%=job%>" property="itemSubtype.name" nulltext="" />
				</td>
			</tr>
			<tr>
				<th>User</th>
				<td>
					<base:propertyvalue item="<%=job%>" property="owner.name" />
				</td>
			</tr>
			<tr>
				<th>Active project</th>
				<td>
					<%=activeProject == null ? (readActiveProject ? "<i>- none -</i>" : "<i>- denied -</i>") : HTML.encodeTags(activeProject.getName()) %>
				</td>
			</tr>
			<tr>
				<th>Experiment</th>
				<td>
					<base:propertyvalue item="<%=job%>" property="experiment.name" />
				</td>
			</tr>
			<tr>
				<th>Status</th>
				<td>
					<div class="<%=status == Job.Status.ERROR ? "error messagecontainer" : ""%>" style="margin: 1px 0px;">
					<%=status%><%=job.isDryRun() ? " (dry-run)" : "" %>: 
					<span id="message">
						<%=(status == Job.Status.WAITING || status == Job.Status.PAUSED) && job.getExecuteCommand() != null ? 
							"<span class=\"executecommand\">(Resume from " + job.getExecuteCommand() + ")</span><br>" : ""%>
						<%=HTML.niceFormat(job.getStatusMessage())%></span>
					</div>
				</td>
			</tr>
			<tr>
				<th class="subprompt">- percent complete</th>
				<td>
					<table class="progressbar" style="margin-bottom: 2px;">
					<tr>
					<td>
						<table class="bar" style="width: 200px;">
						<tr>
							<%
							int percent = job.getPercentComplete();
							%>
							<td id="percentDone" class="percentDone" style="width: <%=percent%>%; <%=percent <= 0 ? "display: none;" : ""%>">&nbsp;</td>
							<td id="percentRemain" class="percentRemain" style="width: <%=100-percent%>%; <%=percent == 100 || percent == -1 ? "display: none;" : ""%>">&nbsp;</td>
							<td id="percentUnknown" class="percentUnknown" style="width: 100%; <%=percent != -1 ? "display: none;" : "" %>">unknown</td>
						</tr>
						</table>
					</td>
					<td id="percentText" class="percentText" 
						style="<%=percent < 0 ? "display: none;" : "" %>"><%=percent%>%</td>
						<%
						if (logFile != null)
						{
							%>
							<td style="padding-left: 10px;"><base:button 
								subclass="auto-init"
								data-auto-init="view-file"
								data-file-id="<%=logFile.getId()%>"
								image="view.png" 
								title="View log&hellip;"
								tooltip="View the log file with detailed information about this job"
								/></td>
							<%
						}
						%>
					</tr>
					</table>
				</td>
			</tr>
			<tr>
				<th>Description</th>
				<td><%=HTML.niceFormat(job.getDescription())%></td>
			</tr>
			<tr>
				<th>Created</th>
				<td>
					<%=dateTimeFormatter.format(job.getCreated())%>
				</td>
			</tr>
			<tr>
				<th class="subprompt">- scheduled</th>
				<td>
					<%=dateTimeFormatter.format(job.getScheduled())%>
				</td>
			</tr>
			<tr>
				<th class="subprompt">- priority</th>
				<td>
					<%=job.getPriority()%> (1 = highest, 10 = lowest)
				</td>
			</tr>
			<tr>
				<th>Running time</th>
				<td id="runningTime">
					<%
					Date started = job.getStarted();
					if (started != null)
					{
						Date ended = job.getEnded();
						if (ended == null) ended = new Date();
						long runningTime = ended.getTime() - started.getTime();
						%>
						<%=Values.formatTime(runningTime / 1000)%>
						<%
					}
					%>
				</td>
			</tr>
			<tr>
				<th class="subprompt">- started</th>
				<td>
					<%=dateTimeFormatter.format(job.getStarted())%>
				</td>
			</tr>
			<tr>
				<th class="subprompt">- ended</th>
				<td>
					<%=dateTimeFormatter.format(job.getEnded())%>
				</td>
			</tr>
			<tr>
				<th>Server</th>
				<td>
					<%=HTML.encodeTags(job.getServerAndNode())%>
				</td>
			</tr>
			<tr>
				<th>External ID</th>
				<td>
					<%=HTML.encodeTags(job.getExternalId())%>
				</td>
			</tr>
			<tr>
				<th>Job agent</th>
				<td><%=Base.getEncodedName(agent, !readAgent)%></td>
			</tr>
			<tr class="dynamic">
				<th></th>
				<td></td>
			</tr>
			</table>
			</t:tab>
			<%
			if (job.getStackTrace() != null)
			{
				%>
				<t:tab id="stacktrace" title="Stack trace" helpid="job.view.stacktrace">
					<div class="stacktrace" style="padding: 0.5em;"><%=HTML.encodeTags(job.getStackTrace().replaceAll("\t", "  "))%></div>
				</t:tab>
				<%
			}
			%>
			<t:tab id="parameters" title="Parameters" helpid="job.view.parameters">
				<table class="fullform outlined">
				<tbody class="sectionheader">
					<tr>
						<th colspan="2">Job parameters</th>
					</tr>
				</tbody>
				<tbody>
				<%
				List<String> names = new java.util.ArrayList<String>(jobParameters);
				Collections.sort(names);
				for (String name : names)
				{
					StringBuilder sb = new StringBuilder();
					String displayValue = "";
					String description = "";
					try
					{
						ParameterInfo pi = job.getParameterInfo(name);
						if (pi.getLabel() != null) name = HTML.encodeTags(pi.getLabel());
						description = HTML.encodeTags(pi.getDescription());
						List<?> values = pi.getValuesOrMask();
						int i = 0;
						for (Object value : values)
						{
							if (value != null)
							{
								if (i > 0) sb.append(", ");
								i++;
								if (value instanceof BasicItem)
								{
									sb.append(HTML.encodeTags(getItemName((BasicItem)value)));
								}
								else if (value instanceof Date)
								{
									sb.append(dateFormatter.format((Date)value));
								}
								else
								{
									sb.append(HTML.encodeTags(value.toString()));
								}
							}
						}
						displayValue = sb.toString();
					}
					catch (Throwable ex)
					{
						displayValue = "<i>ERROR: "+ex.getMessage()+"</i>";
					}
					%>
					<tr>
						<th><span title="<%=description%>"><%=name%></span></th>
						<td>
							<%=displayValue%>
						</td>
					</tr>
					<%
				}
				%>
				<%
				if (!readCurrentConfig)
				{
					%>
					</tbody>
					<tbody class="sectionheader">
						<tr>
							<th colspan="2">Plugin configuration parameters</th>
						</tr>
					</tbody>
					<tbody>
					<tr>
						<th></th>
						<td><i>- denied -</i></td>
					</tr>
					<%
				}
				else if (currentConfig != null)
				{
					%>
					</tbody>
					<tbody class="sectionheader">
						<tr>
							<th colspan="2">Plugin configuration parameters</th>
						</tr>
					</tbody>
					<tbody>
					<tr>
						<th><span 
							title="The version of the parmeters used for this job, current version in paranthesis">Parameter version</span></td>
						<td><%=parameterVersion%> (<%=latestParameterVersion %>)</th>
					</tr>
					<%
					names = new java.util.ArrayList<String>(currentConfig.getParameterNames(parameterVersion));
					Collections.sort(names);
					for (String name : names)
					{
						StringBuilder sb = new StringBuilder();
						String displayValue = "";
						String description = "";
						try
						{
							ParameterInfo pi = currentConfig.getParameterInfo(name, parameterVersion);
							if (pi.getLabel() != null) name = HTML.encodeTags(pi.getLabel());
							description = HTML.encodeTags(pi.getDescription());
							List<?> values = pi.getValuesOrMask();
							int i = 0;
							for (Object value : values)
							{
								if (value != null)
								{
									if (i > 0) sb.append(", ");
									i++;
									if (value instanceof BasicItem)
									{
										sb.append(HTML.encodeTags(getItemName((BasicItem)value)));
									}
									else if (value instanceof Date)
									{
										sb.append(dateFormatter.format((Date)value));
									}
									else
									{
										sb.append(HTML.encodeTags(value.toString()));
									}
								}
							}
							displayValue = sb.toString();
						}
						catch (Throwable ex)
						{
							displayValue = "<i>ERROR: "+ex.getMessage()+"</i>";
						}
						%>
						<tr>
							<th><span title="<%=description%>"><%=name%></span></th>
							<td>
								<%=displayValue%>
							</td>
						</tr>
						<%
					}
					%>
					<%
				}
				%>
				<tr class="dynamic">
					<th></th>
					<td></td>
				</tr>
				</tbody>
				</table>
			</t:tab>
			
			<t:tab id="changes" title="Changed items"
				tooltip="Display a log of all modifications made by this job"
				visible="<%=!autoUpdate && ChangeHistoryUtil.showChangeHistoryTab(sc) %>">
				<%
				ItemQuery<ChangeHistory> query = ChangeHistory.getChangesBy(job);
				query.order(Orders.asc(Hql.property("id")));
				query.setMaxResults(1000);
				query.setReturnTotalCount(true);
				ItemResultIterator<ChangeHistory> changes = query.iterate(dc);
				long numChanges = changes.getTotalCount();
				%>
				<tbl:table 
					id="history" columns="all" 
					subclass="fulltable" stickyheaders="<%=null%>" style="border-bottom: 0px;">
					<tbl:columndef id="changeType" title="Change type" />
					<tbl:columndef id="item" title="Item" />
					<tbl:columndef id="info" title="Info" />
					
					<tbl:data style="top: 0px;">
						<tbl:headers>
							<tbl:headerrow>
								<tbl:columnheaders />
							</tbl:headerrow>
						</tbl:headers>
						<tbl:rows>
						<%
						if (numChanges > 1000)
						{
							%>
							<tbl:panel subclass="bg-filled-100">
								<b>NOTE! Listing 1000 of <span title="<%=numChanges %>"><%=Values.formatNumber(numChanges / 1000f, 1, "k") %></span> total changes.</b>
							</tbl:panel>
							<%
						}
						while (changes.hasNext())
						{
							ChangeHistory change = changes.next();
							%>
							<tbl:row>
								<tbl:cell column="changeType"><%=change.getChangeType()%></tbl:cell>
								<tbl:cell column="item"><%=ChangeHistoryUtil.getItem(dc, change, false, false)%></tbl:cell>
								<tbl:cell column="info"><%=HTML.encodeTags(change.getChangeInfo())%></tbl:cell>
							</tbl:row>
							<%
						}
						if (numChanges == 0)
						{
							%>
							<tbl:panel subclass="bg-filled-50">
								<div class="messagecontainer note">
								No log entries found for this job. NOTE! This job may have made 
								other changes that are not recorded by the logging mechanism.
								</div>
							</tbl:panel>
							<%
						}
						%>
						</tbl:rows>
					</tbl:data>
				</tbl:table>
			</t:tab>
			</t:tabcontrol>
	
		<base:buttongroup subclass="dialogbuttons">
			<%
			if (supportsPause)
			{
				%>
				<base:button id="btnPause" title="Pause" image="pause.png" tooltip="Pause the job" />
				<%
			}
			if (supportsAbort)
			{
				%>
				<base:button id="btnAbort" title="Abort&hellip;" image="abort.png" tooltip="Abort the job" />
				<%
			}
			else if (supportsForceError)
			{
				%>
				<base:button id="btnForceError" title="Force error&hellip;" image="bug.png" tooltip="Force update the job status to ERROR" />
				<%
			}
			%>
			<%
			if ((status == Job.Status.PAUSED || status == Job.Status.ERROR) && job.getJobType() == Job.Type.RUN_PLUGIN)
			{
				boolean restartable = !job.getPluginDefinition().supports("net.sf.basedb.core.plugin.NonRestartable");				
				if (restartable)
				{
					%>
					<base:button 
						id="btnRestartJob" 
						title="Restart job" 
						image="refresh.png" 
						tooltip="Try to run this job again with the same parameters"
					/>
					<base:button 
						id="btnReconfigureJob"
						title="Re-configure job"
						image="runplugin.png"
						tooltip="Change the parameters for this job and try again"
						visible="<%=job.hasContext()%>"
					/>
					<%
				}
			}
			if (status == Job.Status.DONE && job.isDryRun())
			{
				%>
				<base:button 
					id="btnRestartJob"
					data-clear-dry-run="1"
					title="Really run" 
					image="refresh.png" 
					tooltip="Run this dry-run job for real"
				/>
				<base:button 
					id="btnReconfigureJob"
					title="Re-configure job"
					image="runplugin.png"
					tooltip="Change the parameters for this job and try again"
					visible="<%=job.hasContext()%>"
				/>
				<%
			}
			%>
			<base:button id="close" title="Close" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
