<%-- $Id: index.jsp 7601 2019-02-22 10:57:51Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Removable"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Trashcan"
	import="net.sf.basedb.core.ItemInUseException"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.SimpleProgressReporter"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.clients.web.util.HTML"
	import="java.util.Enumeration"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.List"
	import="java.util.LinkedList"
	import="java.util.Iterator"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
	private static final ItemContext defaultContext = Base.createDefaultContext("", "type,name");
	private static final Item itemType = Item.SYSTEM;
	private static final String subContext = "trashcan";
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String cmd = request.getParameter("cmd");
final String root = request.getContextPath()+"/";

final String listPage = "list_trash.jsp?ID="+ID;
final String viewPage = "view_item.jsp?ID="+ID;

String forward = null;
String redirect = null;
String message = null;
String notify = request.getParameter("notify");
DbControl dc = null;

try
{
	if (cmd == null || "List".equals(cmd))
	{
		// Display the list page without updatinging the current context
		Base.getAndSetCurrentContext(sc, itemType, subContext, null, defaultContext);
		redirect = listPage;
	}
	else if ("UpdateContext".equals(cmd))
	{
		// Display the list page after updating the current context from the request parameters
		Base.getAndSetCurrentContext(sc, itemType, subContext, pageContext, defaultContext);
		redirect = listPage;
	}
	else if ("LoadContext".equals(cmd))
	{
		// Display the list page after loading a saved context
		int contextId = Values.getInt(request.getParameter("context"));
		Base.loadContext(sc, contextId, defaultContext);
		redirect = listPage;
	}
	else if ("DeleteItemsPermanently".equals(cmd))
	{
		// Delete all selected items on the list page
		Enumeration<String> names = request.getParameterNames();
		List<Removable> items = new LinkedList<Removable>();
		dc = sc.newDbControl();
		while (names.hasMoreElements())
		{
			String name = names.nextElement();
			if (name.startsWith("item:"))
			{
				Item itemType = Item.valueOf(name.substring(5));
				Integer[] ids = Values.getInt(request.getParameterValues(name));
				for (int itemId : ids)
				{
					try
					{
						items.add((Removable)itemType.getById(dc, itemId));
					}
					catch (Throwable t)
					{}
				}
			}
		}
		dc.close();
		
		int numTotal = items.size();
		SimpleProgressReporter progress = new SimpleProgressReporter(null);
		sc.setSessionSetting("progress.trashcan", progress);
		int numRemoved = Trashcan.delete(sc, items, false, progress);
	
		if (numTotal != numRemoved)
		{
			message = (numRemoved == 0 ? "No" : "Only "+numRemoved+" of "+numTotal) + " items could be deleted, because you have no DELETE permission, or the item is used by other items";
		}
		redirect = listPage;
	}
	else if ("DeleteItems".equals(cmd))
	{
		// Mark selected items as removed
		dc = sc.newDbControl();
		int numTotal = 0;
		int numDeleted = 0;

		Enumeration<String> names = request.getParameterNames();
		while (names.hasMoreElements())
		{
			String name = names.nextElement();
			if (name.startsWith("item:"))
			{
				Item itemType = Item.valueOf(name.substring(5));
				Integer[] ids = Values.getInt(request.getParameterValues(name));
				for (int itemId : ids)
				{
					numTotal++;
					try
					{
						BasicItem item = itemType.getById(dc, itemId);
						Removable r = (Removable)item;
						r.setRemoved(true);
						numDeleted++;
					}
					catch (PermissionDeniedException ex)
					{}
				}
			}
		}
		dc.commit();
		if (numTotal != numDeleted)
		{
			message = (numDeleted == 0 ? "No" : "Only "+numDeleted+" of "+numTotal) + " items could be deleted, because you have no DELETE permission";
		}
		Item itemType = Item.valueOf(request.getParameter("item_type"));
		int itemId = Values.getInt(request.getParameter("item_id"));
		
		redirect = viewPage+"&item_type=" + itemType.name() + "&item_id="+itemId;
	}
	else if ("RestoreItems".equals(cmd))
	{
		// Restore all selected items on the list or view page 
		dc = sc.newDbControl();
		int numTotal = 0;
		int numRestored = 0;

		Enumeration<String> names = request.getParameterNames();
		while (names.hasMoreElements())
		{
			String name = names.nextElement();
			if (name.startsWith("item:"))
			{
				Item itemType = Item.valueOf(name.substring(5));
				Integer[] ids = Values.getInt(request.getParameterValues(name));
				for (int itemId : ids)
				{
					numTotal++;
					try
					{
						BasicItem item = itemType.getById(dc, itemId);
						Removable r = (Removable)item;
						r.setRemoved(false);
						numRestored++;
					}
					catch (PermissionDeniedException ex)
					{}
				}
			}
		}
		dc.commit();
		
		if (numTotal != numRestored)
		{
			message = (numRestored == 0 ? "No" : "Only "+numRestored+" of "+numTotal) + " items could be restored, because you have no WRITE permission";
		}

		if (request.getParameter("item_type") != null)
		{
			Item itemType = Item.valueOf(request.getParameter("item_type"));
			int itemId = Values.getInt(request.getParameter("item_id"));
			redirect = viewPage + "&item_type=" + itemType.name() + "&item_id="+itemId;
		}
		else
		{
			redirect = listPage;
		}
	}
	else if ("RestoreItem".equals(cmd))
	{
		// Restore the currently viewed item
		dc = sc.newDbControl();
		Item itemType = Item.valueOf(request.getParameter("item_type"));
		int itemId = Values.getInt(request.getParameter("item_id"));
		String itemsIndexPage = request.getParameter("itemsIndexPage");
		BasicItem item = itemType.getById(dc, itemId);
		Removable r = (Removable)item;
		r.setRemoved(false);
		dc.commit();
		redirect = itemsIndexPage + "?ID=" + ID + "&cmd=ViewItem&item_id=" + itemId;
	}
	else if ("DeleteItem".equals(cmd))
	{
		// Delete the currently view item
		dc = sc.newDbControl();
		Item itemType = Item.valueOf(request.getParameter("item_type"));
		int itemId = Values.getInt(request.getParameter("item_id"));
		BasicItem item = itemType.getById(dc, itemId);
		dc.deleteItem(item);
		dc.commit();
		if (request.getParameter("popup") != null)
		{
			message = itemType.toString() + " deleted";
		}
		else
		{
			redirect = listPage;
		}
	}
	else if ("DeleteAllPermanently".equals(cmd))
	{
		dc = sc.newDbControl();
		List<Removable> items = Trashcan.getItems(dc, (Set<Item>)null, 0, 0);
		dc.close();
		int numTotal = items.size();
		SimpleProgressReporter progress = new SimpleProgressReporter(null);
		sc.setSessionSetting("progress.trashcan", progress);
		int numRemoved = Trashcan.delete(sc, items, false, progress);
		
		if (numTotal != numRemoved)
		{
			message = (numRemoved == 0 ? "No" : "Only "+numRemoved+" of "+numTotal) + " items could be deleted, because you have no DELETE permission, or the item is used by other items";
		}
		redirect = listPage;
	}
	else if ("ViewUsingItems".equals(cmd))
	{
		redirect = viewPage + "&item_type=" + request.getParameter("item_type") + "&item_id="+request.getParameter("item_id");
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
}
finally
{
	if (dc != null) dc.close();
}

if (forward != null)
{
	sc.setSessionSetting("alert-message", message);
	pageContext.forward(forward);
}
else if (redirect != null)
{
	sc.setSessionSetting("alert-message", message);
	response.sendRedirect(redirect);
}
else if (message == null)
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&wait=0" + 
		(notify != null ? "&notify_opener=" + HTML.urlEncode(notify) : ""));
}
else
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&message="+
		HTML.urlEncode(message) + (notify != null ? "&notify_opener=" + HTML.urlEncode(notify) : ""));
}
%>

